#version 150 core

uniform mat4 viewProjectionMatrix;
uniform vec3 color;

uniform int drawBoundingBoxFlag;
uniform vec3 boundingBox[ 8 ];

uniform int drawDirectionFlag;
uniform vec3 position;
uniform vec3 direction;

uniform int drawPointFlag;
uniform vec3 point;

out VERT_OUT {
  vec3 drawColor;

  int  drawBoundingBoxFlag;
  vec4 boundingBox[ 8 ];

  int  drawDirectionFlag;
  vec4 position;
  vec4 direction;

  int  drawPointFlag;
  vec4 point;
}
vert_out;

void main() {
  vert_out.drawColor = color;

  if ( drawBoundingBoxFlag == 1 ) {
    vert_out.drawBoundingBoxFlag = 1;
    // вершины AABB можно нарисовать в геометрическом шейдере при помощи triangle_strip и индексов: 3, 2, 6, 7, 4, 2, 0, 3, 1, 6, 5, 4, 1, 0
    vert_out.boundingBox[ 0 ] = viewProjectionMatrix * vec4( boundingBox[ 0 ], 1.0f );
    vert_out.boundingBox[ 1 ] = viewProjectionMatrix * vec4( boundingBox[ 1 ], 1.0f );
    vert_out.boundingBox[ 2 ] = viewProjectionMatrix * vec4( boundingBox[ 2 ], 1.0f );
    vert_out.boundingBox[ 3 ] = viewProjectionMatrix * vec4( boundingBox[ 3 ], 1.0f );
    vert_out.boundingBox[ 4 ] = viewProjectionMatrix * vec4( boundingBox[ 4 ], 1.0f );
    vert_out.boundingBox[ 5 ] = viewProjectionMatrix * vec4( boundingBox[ 5 ], 1.0f );
    vert_out.boundingBox[ 6 ] = viewProjectionMatrix * vec4( boundingBox[ 6 ], 1.0f );
    vert_out.boundingBox[ 7 ] = viewProjectionMatrix * vec4( boundingBox[ 7 ], 1.0f );
  }

  if ( drawDirectionFlag == 1 ) {
    vert_out.drawDirectionFlag = 1;
    vert_out.position          = viewProjectionMatrix * vec4( position, 1.0f );
    vert_out.direction         = viewProjectionMatrix * vec4( direction, 1.0f );
  }

  if ( drawPointFlag == 1 ) {
    vert_out.drawPointFlag = 1;
    vert_out.point         = viewProjectionMatrix * vec4( point, 1.0f );
  }
}
