#version 150 core

uniform bool homogeneousReversedDepth;
uniform sampler2D prevMipmap;
uniform ivec2 divisionFlags;

in vec2 textureCoord;

float getSupremeValue( in vec2 value ) {
  if ( homogeneousReversedDepth ) {
    return min( value.x, value.y );
  }
  return max( value.x, value.y );
}
float getSupremeValue( in vec3 value ) {
  if ( homogeneousReversedDepth ) {
    return min( value.x, min( value.y, value.z ) );
  }
  return max( value.x, max( value.y, value.z ) );
}
float getSupremeValue( in vec4 value ) {
  if ( homogeneousReversedDepth ) {
    return min( min( value.x, value.y ), min( value.z, value.w ) );
  }
  return max( max( value.x, value.y ), max( value.z, value.w ) );
}

void main() {
  const ivec2 topLeft     = ivec2( -1, 1 );
  const ivec2 top         = ivec2( 0, 1 );
  const ivec2 topRight    = ivec2( 1, 1 );
  const ivec2 right       = ivec2( 1, 0 );
  const ivec2 rightBottom = ivec2( 1, -1 );
  const ivec2 bottom      = ivec2( 0, -1 );
  const ivec2 leftBottom  = ivec2( -1, -1 );
  const ivec2 left        = ivec2( -1, 0 );

  vec4 texels;
  texels.x    = texture( prevMipmap, textureCoord ).r;
  texels.y    = textureOffset( prevMipmap, textureCoord, top ).r;
  texels.z    = textureOffset( prevMipmap, textureCoord, topRight ).r;
  texels.w    = textureOffset( prevMipmap, textureCoord, right ).r;
  float depth = getSupremeValue( texels );

  // если ширина или высота не делится на 4, делаем дополнительные выборки
  if ( divisionFlags.x != 1 ) {
    if ( divisionFlags.y != 1 ) {
      vec2 extra;
      extra.x = depth;
      extra.y = textureOffset( prevMipmap, textureCoord, leftBottom ).r;
      depth   = getSupremeValue( extra );
    }
    vec3 extra;
    extra.x = depth;
    extra.y = textureOffset( prevMipmap, textureCoord, left ).r;
    extra.z = textureOffset( prevMipmap, textureCoord, topLeft ).r;
    depth   = getSupremeValue( extra );
  }
  if ( divisionFlags.y != 1 ) {
    vec3 extra;
    extra.x = depth;
    extra.y = textureOffset( prevMipmap, textureCoord, bottom ).r;
    extra.z = textureOffset( prevMipmap, textureCoord, rightBottom ).r;
    depth   = getSupremeValue( extra );
  }
  gl_FragDepth = depth;
}
