#version 150 core
#extension GL_ARB_explicit_attrib_location : enable

layout( location = 0 ) in vec3 inVertex;
layout( location = 1 ) in vec3 inTextureCoord;
layout( location = 2 ) in vec3 inNormal;
layout( location = 3 ) in vec3 inTangent;

uniform mat4 modelViewProjectionMatrix;
uniform mat3 normalMatrix;

uniform mat4 modelMatrix;
uniform mat4 viewProjectionMatrix;
uniform vec3 wsCameraPosition;

out vec2 textureCoord;
out mat3 tbnMatrix;
out mat4 tangentToClipMatrix;

out vec3 wsPosition;
out vec3 tsPosition;
out vec3  tsCameraPosition;
out float distanceToFragment;

void main() {
  gl_Position  = modelViewProjectionMatrix * vec4( inVertex, 1.0f );
  textureCoord = inTextureCoord.xy;

  // создаем матрицу перевода направления в координаты пространства мира
  vec3 wsNormal       = normalize( normalMatrix * inNormal );
  vec3 wsTangent      = normalize( normalMatrix * inTangent );
  vec3 wsBitangent    = normalize( cross( wsNormal, wsTangent ) );
  tbnMatrix           = mat3( wsTangent, wsBitangent, wsNormal );
  tangentToClipMatrix = viewProjectionMatrix * mat4( tbnMatrix );

  // положение камеры и фрагмента в координатах касательного пространства полигона (tangent space)
  mat3 ws2tsMatrix = inverse( tbnMatrix );
  wsPosition       = vec3( modelMatrix * vec4( inVertex, 1.0f ) );
  tsPosition       = ws2tsMatrix * wsPosition;
  tsCameraPosition = ws2tsMatrix * wsCameraPosition;

  distanceToFragment = length( wsPosition - wsCameraPosition );
}
