#version 120

varying vec2 textureCoord;
varying mat3 tbnMatrix;

uniform sampler2D textureAlbedo;
uniform sampler2D textureNormal;
uniform vec3  screenGamma;
uniform float exposure;
uniform float alphaTestThreshold;
uniform float transparency;
uniform float selfIllumination;

const vec3  lightColor = vec3( 1.0f, 1.0f, 1.0f );
const vec3  wsLightDir = normalize( vec3( -1.0f, 1.0f, 1.0f ) );
const float ambient    = 0.05f;

vec3 getNormal( in vec2 texCoord ) {
  // переводим из tangent space в NDC [-1.0f, 1.0f]
  vec3 ndcNormal = texture2D( textureNormal, texCoord ).xyz * 2.0f - 1.0f;
  // переводим в мировое пространство
  return normalize( tbnMatrix * ndcNormal );
}

void main() {
  float diffuse     = max( dot( getNormal( textureCoord ), wsLightDir ), 0.0f );
  vec3  light       = max( ( diffuse + ambient ), selfIllumination ) * lightColor;
  vec4  colorLinear = texture2D( textureAlbedo, textureCoord ).rgba;
  float alpha       = min( colorLinear.a, transparency );
  if ( alpha < alphaTestThreshold ) {
    discard;
  }
  vec3 finalColor = light * colorLinear.rgb;
  vec3 mapped     = vec3( 1.0f ) - exp( -finalColor * exposure );

  gl_FragColor = vec4( pow( mapped, screenGamma ), alpha );
}
