#version 120

varying vec2 textureCoord;

uniform sampler2D albedo;
uniform vec3 screenGamma;
uniform bool isColor;
uniform vec4  color;
uniform bool  isFont;
uniform float smoothing;
uniform bool  hasOutline;
uniform float outlineWidth;
uniform vec3 outlineColor;

const float exposure    = 1.05f;
const float symbolWidth = 0.5f;

void main() {
  vec4 resultColor = color;
  if ( isColor && isFont ) {
    float distance = texture2D( albedo, textureCoord ).a;
    float alpha    = smoothstep( symbolWidth - smoothing, symbolWidth + smoothing, distance );
    if ( hasOutline ) {
      resultColor = mix( vec4( outlineColor, 1.0f ), resultColor, alpha );
      alpha       = smoothstep( outlineWidth - smoothing, outlineWidth + smoothing, distance );
    }
    resultColor = vec4( resultColor.rgb, alpha );
  } else if ( isColor ) {
    resultColor = color;
  } else {
    resultColor = texture2D( albedo, textureCoord );
  }

  // Exposure tone mapping
  vec3 mapped  = vec3( 1.0f ) - exp( -resultColor.rgb * exposure );
  gl_FragColor = vec4( pow( mapped, screenGamma ), resultColor.a );
}
